#!/usr/bin/env python
# coding: utf-8
#
#       Copyright 2008 Olivier Berten <olivier.berten@gmail.com>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#

import os
import sys
from color import *

# from http://code.djangoproject.com/browser/django/trunk/django/utils/datastructures.py
class SortedDict(dict):
	"""
	A dictionary that keeps its keys in the order in which they're inserted.
	"""
	def __new__(cls, *args, **kwargs):
		instance = super(SortedDict, cls).__new__(cls, *args, **kwargs)
		instance.keyOrder = []
		return instance

	def __init__(self, data=None):
		if data is None:
			data = {}
		super(SortedDict, self).__init__(data)
		if isinstance(data, dict):
			self.keyOrder = data.keys()
		else:
			self.keyOrder = []
			for key, value in data:
				if key not in self.keyOrder:
					self.keyOrder.append(key)

	def __deepcopy__(self, memo):
		from copy import deepcopy
		return self.__class__([(key, deepcopy(value, memo))
							   for key, value in self.iteritems()])

	def __setitem__(self, key, value):
		super(SortedDict, self).__setitem__(key, value)
		if key not in self.keyOrder:
			self.keyOrder.append(key)

	def __delitem__(self, key):
		super(SortedDict, self).__delitem__(key)
		self.keyOrder.remove(key)

	def __iter__(self):
		for k in self.keyOrder:
			yield k

	def pop(self, k, *args):
		result = super(SortedDict, self).pop(k, *args)
		try:
			self.keyOrder.remove(k)
		except ValueError:
			# Key wasn't in the dictionary in the first place. No problem.
			pass
		return result

	def popitem(self):
		result = super(SortedDict, self).popitem()
		self.keyOrder.remove(result[0])
		return result

	def items(self):
		return zip(self.keyOrder, self.values())

	def iteritems(self):
		for key in self.keyOrder:
			yield key, super(SortedDict, self).__getitem__(key)

	def keys(self):
		return self.keyOrder[:]

	def iterkeys(self):
		return iter(self.keyOrder)

	def values(self):
		return [super(SortedDict, self).__getitem__(k) for k in self.keyOrder]

	def itervalues(self):
		for key in self.keyOrder:
			yield super(SortedDict, self).__getitem__(key)

	def update(self, dict_):
		for k, v in dict_.items():
			self.__setitem__(k, v)

	def setdefault(self, key, default):
		if key not in self.keyOrder:
			self.keyOrder.append(key)
		return super(SortedDict, self).setdefault(key, default)

	def value_for_index(self, index):
		"""Returns the value of the item at the given zero-based index."""
		return self[self.keyOrder[index]]

	def insert(self, index, key, value):
		"""Inserts the key, value pair before the item with the given index."""
		if key in self.keyOrder:
			n = self.keyOrder.index(key)
			del self.keyOrder[n]
			if n < index:
				index -= 1
		self.keyOrder.insert(index, key)
		super(SortedDict, self).__setitem__(key, value)

	def copy(self):
		"""Returns a copy of this object."""
		# This way of initializing the copy means it works for subclasses, too.
		obj = self.__class__(self)
		obj.keyOrder = self.keyOrder[:]
		return obj

	def __repr__(self):
		"""
		Replaces the normal dict.__repr__ with a version that returns the keys
		in their sorted order.
		"""
		return '{%s}' % ', '.join(['%r: %r' % (k, v) for k, v in self.items()])

	def clear(self):
		super(SortedDict, self).clear()
		self.keyOrder = []

class SwatchBook(object):
	"""Output values
       RGB,HSV,HSL,CMY,CMYK,6CLR: 0 -> 1
       YIQ: Y 0 -> 1 : IQ -0.5 -> 0.5
       Lab: L 0 -> 100 : ab -128 -> 127
       XYZ: 0 -> ~100 (cfr. ref)"""

	def __init__(self, file=False):
		# Informations
		self.info = {}
		self.inks = False
		# Display informations
		self.display = {}
		# Color Profiles
		self.profiles = {}
		# Swatches
		self.items = SortedDict()

		if file:
			self.read(file)

	def read(self,file):
		if not os.path.isfile(file):
			sys.stderr.write('file '+file+' doesn\'t exist')
			return
		if os.path.getsize(file) == 0:
			sys.stderr.write('empty file')
			return
		import swatchbook.codecs as codecs
		ext =  os.path.splitext(os.path.basename(file))[1].lower()[1:]
		codec = False
		if ext in codecs.readexts:
			for codec in codecs.readexts[ext]:
				test = eval('codecs.'+codec).test(file)
				if test: break
			else:
				codec = False
		if codec:
			eval('codecs.'+codec).read(self,file)
			self.info['filename'] =  os.path.splitext(os.path.basename(file))[0]
			# Don't know if this is the right way...
			if isinstance(self.info['filename'],str):
				self.info['filename'] = unicode(self.info['filename'],'utf-8')
			if 'name' not in self.info:
				self.info['name'] = {0: self.info['filename'].replace('_',' ')}

		else:
			sys.stderr.write(file.encode('utf-8')+': unsupported input format\n')

	def write(self,format,output=None):
		import swatchbook.codecs as codecs
		if format in codecs.writes:
			codec = eval('codecs.'+format)
			if output == None:
				print codec.write(self)
			else:
				content = codec.write(self)
				# TODO check if writable
				bookfile = open(output, 'w')
				bookfile.write(content)
				bookfile.close()
		else:
			sys.stderr.write('unsupported output format\n')

class Group(object):
	def __init__(self):
		self.id = False
		self.info = {}
		self.items = SortedDict()

class Swatch(object):
	def __init__(self):
		self.id = False
		self.info = {}

class Spacer(object):
	def __init__(self):
		self.id = False

class Break(object):
	def __init__(self):
		self.id = False

class Color(Swatch):
	def __init__(self):
		super(Color, self).__init__()
		self.values = {}
		self.attr = []

	def toRGB8(self):
		values = unicc(self.values.copy())
		if 'RGB' in values:
			R,G,B = values['RGB']
		elif 'HSV' in values:
			H,S,V = values['HSV']
			R,G,B = HSV2RGB(H,S,V)
		elif 'HSL' in values:
			H,S,L = values['HSL']
			R,G,B = HSL2RGB(H,S,L)
		elif 'CMY' in values:
			C,M,Y = values['CMY']
			R,G,B = CMY2RGB(C,M,Y)
		elif 'YIQ' in values:
			Y,I,Q = values['YIQ']
			R,G,B = YIQ2RGB(Y,I,Q)
		elif 'Lab' in values:
			L,a,b = values['Lab']
			R,G,B = Lab2RGB(L,a,b)
		elif 'CMYK' in values:
			C,M,Y,K = values['CMYK']
			R,G,B = CMYK2RGB(C,M,Y,K)
		elif 'Gray' in values:
			R = G = B = 1-values['Gray'][0]
		else:
			return False
		return (int(round(R*0xFF)),int(round(G*0xFF)),int(round(B*0xFF)))
			
class Tint(Swatch):
	def __init__(self):
		super(Tint, self).__init__()
		self.color = False
		self.density = 1 # 0 = white

class Tone(Swatch):
	def __init__(self):
		super(Tone, self).__init__()
		self.color = False
		self.density = 1 # 0 = gray

class Shade(Swatch):
	def __init__(self):
		super(Shade, self).__init__()
		self.color = False
		self.density = 1 # 0 = black

class Gradient(Swatch):
	def __init__(self):
		super(Gradient, self).__init__()
		self.stops = []

class Stop(object):
	def __init__(self):
		pass

class Pattern(Swatch):
	def __init__(self):
		super(Pattern, self).__init__()
		self.colors = []

